#!/usr/bin/perl
#                   -w                      # for debugging

# A horizontal sundial with a longitude correction, text and graphics, and text popup box
# www.illustratingshadows.com               # January 8, 2013

# http://www.tkdocs.com/tutorial
# http://www.tkdocs.com/tutorial/firstexample.html#code

#                   # http://perldoc.perl.org/Math/Trig.html#TRIGONOMETRIC-FUNCTIONS
use Math::Trig;     # needs "use Math::Trig;" as "atan2" is worthless
                    # also TAN is in math.trig, otherwise you must
                    # do sin/cos. But the big deal is TAN vs ATAN2

use TKx;            # needed for graphics
# use Tk;

tk::toplevel .t ;   # create a top level window

# $t = TkToplevel.new(parent);

use Tkx;

# ##########################
# # WHERE INPUT HAPPENS    #
# ##########################

Tkx::wm_title  (".", "LOCATION");
Tkx::ttk__frame(".c" ,  -padding => "60 15 20 20" );
Tkx::grid( ".c", -column => 0, -row => 0, -sticky => "nwes");
Tkx::grid_columnconfigure( ".", 0, -weight => 1);
Tkx::grid_rowconfigure   ( ".", 0, -weight => 1);

Tkx::ttk__entry (".c.lat",   -width  => 7, -textvariable => \$lat);
Tkx::grid       (".c.lat",   -column => 2, -row => 1, -sticky => "we");
Tkx::ttk__entry (".c.lng",   -width  => 7, -textvariable => \$lng);
Tkx::grid       (".c.lng",   -column => 2, -row => 2, -sticky => "we");
# Tkx::ttk__label (".c.meters", -textvariable => \$meters);
# Tkx::grid       (".c.meters", -column => 2, -row => 2, -sticky => "we");
Tkx::ttk__button(".c.calc",   -text => "DRAFT", -command => sub {calculate();});
Tkx::grid       (".c.calc",   -column => 3, -row => 3, -sticky => "w");
Tkx::ttk__button(".c.quit",   -text => "QUIT", -command => sub {quitNow();});
Tkx::grid       (".c.quit",   -column => 3, -row => 4, -sticky => "w");

Tkx::grid( Tkx::ttk__label(".c.latlbl",  -text => "latitude"),  -column => 3, -row => 1, -sticky => "w");
Tkx::grid( Tkx::ttk__label(".c.lnglbl",  -text => "longitude"), -column => 3, -row => 2, -sticky => "w");


foreach (Tkx::SplitList(Tkx::winfo_children(".c"))) {
    Tkx::grid_configure($_, -padx => 5, -pady => 5);
}
Tkx::focus(".c.lat");                              ################################
Tkx::bind (".", "<Return>", sub {calculate();});   # invokes the subroutine below #
                                                   ################################

# ##########################################################################################
# # WHERE THE WORK IS DONE # invoked by Tkx::bind (".", "<Return>", sub {calculate();});   #
# ##########################################################################################
sub calculate {
   Tkx::grid( Tkx::ttk__label(".c.info",  -text => "OK"),       -column => 2, -row => 3, -sticky => "w");
   
   #############################
   $longitude = $lng;
   $latitude  = $lat;
   $slat      = sin($lat*2*3.14159/360);
   
   $mw = Tkx::widget->new(".");
   # http://www.tkdocs.com/tutorial/canvas.html
   $canvas = $mw->new_tk__canvas(-width => 800, -height => 600,
                                 -background => "yellow" ) ;
   $canvas->g_grid(-column=>0, -row=>0);

   $canvas->create_text(200,20, -text=>"HORIZONTAL SUNDIAL",-anchor=>"e");
   $canvas->create_text(500,20, -text=>"www.illustratingshadows.com",-anchor=>"e");
   $canvas->create_text(200,40, -text=>"LATITUDE",-anchor=>"e");
   $canvas->create_text(300,45, -text=>$latitude,-anchor=>"e");
   $canvas->create_text(200,60, -text=>"LONGITUDE OFFSET",-anchor=>"e");
   $canvas->create_text(300,60, -text=>-$longitude,-anchor=>"e");
   $canvas->create_text(600,80, -text=>"**** IGNORE PERL ABNORMAL TERMINATION ****",-anchor=>"e");
   $canvas->create_text(600,95, -text=>"WHEN YOU CLICK THE RED X IN THE GRAPH AREA",-anchor=>"e");
   $canvas->create_text(600,110,-text=>"CLICK THE RED X IN DOS TEXT AREA INSTEAD  ",-anchor=>"e");
   

   $hr=-5;
   print "\n*** IGNORE PERL ABNORMAL TERMINATION ***";
   while ($hr<6) {                              # NOTE: ctrl-C will break a loop in Windows 8

       # following does not need "use Math::Trig;" however, it is needed for ATAN etc
       # $hlaTAN = $slat ;
       # $hlaTAN = $hlaTAN * sin(15*($hr+($longitude*4/60)))*2*3.14159/360) ;
       # $hlaTAN = $hlaTAN / cos(15*($hr+($longitude*4/60)))*2*3.14159/360) ;

       $hlaTAN = $slat * Math::Trig::tan(15*($hr+($longitude*4/60))*2*3.14159/360);

       $hla    = atan($hlaTAN);                 # needs "use Math::Trig;" as "atan2" is worthless
       $hla    = $hla*360/(2*3.14159);

       # http://alvinalexander.com/blog/post/perl/reference-page-perl-printf-formatting-format-cheat-sheet
       print "\nhour is: " , $hr+12 ," hour line angle is: ";
       printf("%.2f", $hla);
       $thla=sprintf("%.2f", $hla);
       #
       $xc=400;
       $yc=500;
       $radial = 350;
       $y=$radial*cos($hla*2*3.14159/360);
       $x=$radial*sin($hla*2*3.14159/360);
       $canvas->create_line($xc,$yc,   $xc+$x,$yc-$y);
       $canvas->create_text($xc+$x,$yc-$y,-text=>$thla);
       $canvas->create_text($xc+$x,$yc+10-$y,-text=>(12+$hr));

       $hr = $hr + 1;
   }
   


   # draw gnomon
   $canvas->create_line($xc,$yc,         $xc+0,$yc-200);
   $x = 200*sin($latitude*2*3.14159/360);
   $canvas->create_line($xc,$yc-200,     $xc+$x,$yc-200);
   $canvas->create_line($xc,$yc,         $xc+$x,$yc-200);
   $canvas->create_text($xc+$x+10,$yc-200,-text=>"LAT:",-anchor=>"e");
   $canvas->create_text($xc+$x+30,$yc-200,-text=>$latitude,-anchor=>"e");
   
#############################

}


# THIS WORKS FOR THE BUTTON TO QUIT
sub quitNow {
   Tkx::exit(1);
}


# THIS IS WHAT RUNS THE TKX SYSTEM
Tkx::MainLoop();

# END #


